<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class Admin_SiteController extends Controller
{   
    /**
    * set access privileges
    */       
    function init()
    {        
        $this->acl->allow('administrator', $this->name, "*");
         
        if(!$this->acl->isAllowed($this->session->get("role"), $this->name, $this->action))
        {
            $this->redirect(FrontController::getRewrittedUrl("/admin/main/logIn"));  
        }
    }   
    
    /**
    * Delete site
    */
    function deleteAction($siteId, $categoryRedirect = false)
    {
        $referer = $this->request->getRefererUrl();
        
        if($categoryRedirect)
        {
            $site = $this->site->findByPk($siteId);
            
            if(!empty($site))
            {
                $referer = FrontController::getRewrittedUrl("/admin/category/index/".$site->categoryId);
            }
        }
        
        $this->site->delByPk($siteId);
        
        $this->redirect($referer);
    }
    
    /**
    * Update site state from waiting sites
    */
    function updateSiteStateAction()
    {
        //check that siteid is set and status is valid
        if(!empty($this->request->siteIds)
        && in_array($this->request->status, array("validated", "banned", "refused")))
        {
            $siteIds = $this->request->siteIds;
            $status = $this->request->status;
            
            $mailer = Mailer::getInstance();
            
            //get sites which status code should be changed
            $c = new Criteria();
            $c->add("siteId", $siteIds, "IN");
            $sites = $this->site->findAll($c, "*", true);

            foreach($sites as $site)
            { 
                $site['status'] = $status;
                $customSubject = false;
                $customDescription = false;
                $siteId = $site['siteId'];
                
                //if admin select custom message copy it to mail message
                if($this->request->emailType[$siteId] == "custom")
                {
                    $customSubject = $this->request->subject[$siteId];
                    $customDescription = $this->request->description[$siteId];
                }
                
                //send mail with state update
                $mailer->sendSiteStateUpdate($site['webmasterEmail'], $site, $customSubject, $customDescription);      
            }
                
            if($status == "refused")
            {
                //delete site
                $this->site->del($c);
                
                //update refusing stats
                $this->refusal->updateRefusedSitesCount(0, count($sites)); 
            }
            else
            {
                //set page moderator id 0 = admin
                $this->site->update(array("status" => $status, "moderatorId" => 0, "_creationDate" => "NOW()"), $c);
            }
        }
                       
        $this->redirect($this->moduleLink("waiting"));        
    }
    
    /**
    * Display waiting pages
    */
    function waitingAction()
    {
        $emailMessages = $this->customMessage->getUserDefinedMessages(); 
        $this->set("emailMessages", $emailMessages);
        $this->set("emailMessagesJSON", JsonView::php2js($emailMessages));
        
        $this->set("sites", $this->site->getWaitingSites());
    }
    
    /**
    * Get webmaster data when site is editing and we change webmaster
    */
    function getWebmasterDataAction()
    {
        $c = new Criteria();
        $c->add("userId", $this->request->webmasterId);
        $c->add("role", "webmaster");
        $site = $this->user->find($c, "login, email");  
        $this->set($site->toArray());
        $this->viewClass = "JsonView";
    }
    
    /**
    * Display site edit form
    */
    function editAction($siteId)
    {   
        $this->set("siteId", $siteId);
        $site = $this->site->findByPk($siteId); 
        if(empty($site))
        {
            return $this->return404();
        }
        
        $site->updateImageSrc();
        $this->set("site", $site);
        
        $c = new Criteria();
        $c->add("siteId", $siteId);
        $this->set("comments", $this->comment->findAll($c));
        
        $this->set("selectCategories", $this->category->createOptionsList());
        $c = new Criteria();
        $c->add("role", "webmaster");
        $this->set("webmasters", $this->user->generateList($c, "email"));
        
        $this->set("allKeywordsList", $this->keyword->generateSortedList());
        $this->set("yesNoOptions", array("0" => _t("No"), "1" => _t("Yes")));
        $this->set("priorites", range(0, 10));
        
        $this->set("adCriterias", $this->adCriteria->getSelectList());
                
        require(CODE_ROOT_DIR."config/flags.php");
        $this->set("countryFlags", $countryFlags); 

    }
    
    function getMetaDataAction()
    {
        $httpClient = new HttpClient();
        $metaData = $httpClient->getMetaValues($this->request->url);
        $this->set($metaData);
        $this->viewClass = "JsonView";    
    }
    
    function getWebsiteDataAction()
    {
        $siteId = $this->request->siteId;
        $site = $this->site->findByPk($siteId);
        
        $site->keywordIds = $site->getKeywordIds();   
        $this->set($site->toArray());
        $this->viewClass = "JsonView";
    }
    
    function problemAction()
    {
        $c = new Criteria();
        $c->add("problemExists", 1);
        $this->set("sites", $this->site->findAll($c));
    }
    
    function searchAction()
    {
        $sites = array();

        if(!empty($this->request->searchedUrl))
        {      
            $sites = $this->site->search($this->request->searchedUrl);
        }

        $this->set("sites", $sites);
    }
    
    function errorCodeAction()
    {
        $c = new Criteria();
        $c->add("httpCode IS NOT NULL");
        $c->add("httpCode", 200, "!=");
        $this->set("sites", $this->site->findAll($c, "siteId, siteTitle, httpCode"));
    }
    
    function saveNewAction()
    {
        $this->viewClass = "JsonView";

        $errorMessage = $this->site->validate($this->request, null, !empty($this->request->forceCategoryDuplicate));
        
        if($errorMessage)
        {
            switch($errorMessage)
            {
                case "The site is already registered in this category.": 
                    $errorMessage = "The site is already registered in this category. Force add?";
                    $status = "confirm";
                    break;
                  
                default: 
                    $status = "error";
            
            }
            
            $this->set("status", $status);
            $this->set("message", _t($errorMessage));
            return;
        }

        $fields = array("webmasterName", "webmasterEmail", "siteTitle", "url", "rssTitle", "rssFeedOfSite",
                        "description", "returnBond", "status", "priority", "categoryId");
                        
        if(Config::get("countryFlagsEnabled"))
        {
            array_push($fields, "countryCode");
        }
        
        $site = new SiteRecord($this->request->getArray($fields));
        $site->description = substr($site->description, 0, Config::get('siteDescriptionMaxLength'));
        $site->status = "validated";       
        $site->save();

        // store keywords
        if(!empty($this->request->keywords))
        {
            $this->keywordsOfSite->storeKeywords($site->siteId, $this->request->keywords);
        }
        
        $this->set("status", "ok");
        $this->set("refresh", true);
        $this->set("message", _t("The site was successfully submitted."));
    }
    
    function saveSiteDataAction($siteId)
    {
        $this->viewClass = "JsonView";
        
        $fields = array("categoryId", "webmasterName", "webmasterEmail", "siteTitle", "url", "rssTitle", "rssFeedOfSite",
                        "description", "returnBond", "status", "problemExists", "priority", "visitsCount", "referrerTimes");

        if(Config::get("countryFlagsEnabled"))
        {
            array_push($fields, "countryCode");
        }
        
        $data = $this->request->getArray($fields);
        $site = $this->site->findByPk($siteId);
        
        $errorMessage = $this->site->validate($this->request, $site);

        if($errorMessage)
        {
            $this->set("message", $errorMessage);
            $this->set("status", "error");
            return;
        }
        
        $site->fromArray($data);
        
        $site->description = substr($site->description, 0, Config::get('siteDescriptionMaxLength'));       
        $site->save();

        // store keywords
        if(!empty($this->request->keywords))
        {
            $this->keywordsOfSite->storeKeywords($site->siteId, $this->request->keywords);
        }
        
        $this->set("message", "Updated");
        $this->set("status", "ok");
    }
    
    function uploadSiteImageAction($siteId)
    {
        $siteId = intval($siteId);
        $site = $this->site->findByPk($siteId);
        
        $siteImage = new UploadedFile("siteImage");
        $siteImage->addFilter("extension", array("jpg", "png", "gif"));
        $thumbsPath = Config::get("SITES_THUMBS_PATH");
        $newImageSrc = $siteImage->save($thumbsPath);
        
        if($newImageSrc)
        {
            if($site->imageSrc)
            {
                $oldFilePath = $thumbsPath . basename($site->imageSrc);
                
                if(file_exists($oldFilePath))
                {
                    unlink($oldFilePath);
                }
            }
            
            $site->imageSrc = $newImageSrc;
            $site->save();
        }
        
        $site->updateImageSrc();
        
        echo "<html><body>".$site->imageSrc."</body></html>";
        $this->autoRender = false;
    }
    
} 

?>