<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

class SiteModel extends Model
{   
    protected $primaryKey = "siteId";
    
    public function getIfHostIsBanned($hostname, $exceptSiteId = 0)
    {
        $hostname = addslashes($hostname);
        $c = new Criteria();
        $c->add("status", "banned");
        $c->add("siteId", $exceptSiteId);
        $c->add("LOCATE( '$hostname', `url` ) = 1 ");
        return $this->find($c) ? true : false;
    }
    
    public function getHostNameFromUrl($url)
    {
        $regs = array();
        
        $hostname = '';
        
        if (ereg('^http://([^/]*)$', $url))
            $hostname = $url;
        else
        {
            ereg('^http://([^/]*)/(.*)$', $url, $regs);
            $hostname = 'http://' . $regs[1];
        }
            
        return $hostname;
    }
    
    public function getCountOfSiteCopiesInDifferentCategories($url)
    {
        $hostname = $this->getHostNameFromUrl($url);
        $hostname = addslashes($hostname);
        $c = new Criteria();
        $c->add("LOCATE( '$hostname', `url` ) = 1 ");
        
        return $this->getCount($c);
    }
    
    public function whetherSiteIsRegisteredInCategory($url, $categoryId)
    {
        $hostname = addslashes($this->getHostNameFromUrl($url));
        
        $c = new Criteria();
        $c->add("categoryId", $categoryId);
        $c->add("LOCATE( '$hostname', `url` ) = 1 ");
        
        return ($this->getCount($c) > 0);
    }
    
    function selectWithNewFlag(Criteria $c, $fields = "*")
    {
        $noveltyPeriod = intval(Config::get("siteNoveltyPeriod"));
    
        $sites = $this->findAll($c, $fields.", ADDDATE(creationDate, $noveltyPeriod) > NOW() as isNew", true);
 
        foreach($sites as $site)
        {
            $site->updateImageSrc();
        }
        
        return $sites;
    }
    
    function getWaitingSites($siteId = false)
    {
        $sites = array();
        $c = new Criteria();
        $c->add("status", "waiting");
        $c->addOrder("returnBond = '' ASC");
        $c->addOrder("creationDate");
        
        if($siteId)
        {
            $c->add("siteId", $siteId, is_array($siteId) ? "IN" : "=");
        }
        
        $sites = $this->site->selectWithNewFlag($c);
        
        foreach($sites as $site)
        {
            $site->categoryParentsData = $this->categoryParent->getParentsData($site->categoryId);
            $site->keywords = $site->getKeywords();
        }
        
        return $sites;
    }
    
    function generateNewSelection()
    {         
        $c = new Criteria();
        $c->add("status", "validated");
        $c->addOrder("RAND()");
        $c->setLimit(Config::get('selectionSitesCount'));  
        
        $siteIds = $this->site->generateList($c, "siteId");
        $siteIds = array_keys($siteIds);

        $this->setting->set('selectionSiteIds', implode(",", $siteIds));
        
        $todayDate = date('Y-m-d');                     
        $this->setting->set('selectionGenerationDate', $todayDate);
        
        $cache = Cacher::getInstance();
        $cache->clean("tag", array("site", "setting"));

        return $siteIds;
    }
    
    function getIndexRandomList($refreshNeeded)
    {
        if(!$refreshNeeded)
        {
            $siteIds = explode(",", Config::get("selectionSiteIds"));
        }
        
        if($refreshNeeded)
        {
            $siteIds = $this->generateNewSelection();
        }
        
        if(empty($siteIds))return array();

        $c = new Criteria();  
        $c->add("siteId", $siteIds, "IN");
        
        $sites = $this->selectWithNewFlag($c);
        
        foreach($sites as $key => $site)
        {
            $sites[$key] = $site->toArray();
        }
        
        return $sites;
    }
                
    function getNewValidatedSites()
    {
        $noveltyPeriod = intval(Config::get("siteNoveltyPeriod"));
        
        $c = new Criteria();
        $c->add("ADDDATE(`creationDate`, $noveltyPeriod) > NOW()");
        $c->add("status", "validated");
        $c->addOrder("creationDate DESC");
        
        $fields = "url, siteTitle, siteId, imageSrc, description, creationDate, countryCode";
        return $this->selectWithNewFlag($c, $fields);
    }
    
    function getValidatedTopVisitedSites()
    {
        $c = new Criteria();
        $c->add("visitsCount", 0, ">");
        $c->add("status", "validated");
        $c->addOrder("visitsCount DESC");
        $c->setLimit(Config::get('maxTopHitsCount'));
        
        return $this->selectWithNewFlag($c, "siteId, url, imageSrc, siteTitle, description, visitsCount, countryCode");
    }
    
    function getValidatedTopElectedSites()
    {
        $c = new Criteria();
        $c->add("votesAverage", 0, ">");
        $c->add("status", "validated");
        $c->addOrder("votesAverage DESC");
        $c->setLimit(Config::get('maxTopNotesCount'));
        
        return $this->selectWithNewFlag($c, "siteId, url, imageSrc, siteTitle, description, votesAverage, votesCount, countryCode");  
    }
    
    function getValidatedTopRankSites($pageRank = false)
    {
        $c = new Criteria();
        $c->add("status", "validated");                                            
        $c->addInnerJoin("cachegoogledetails", "cachegoogledetails.url", "sites.url");
        $c->addOrder("pageRank DESC");
        $c->addOrder("siteId DESC");
        
        if($pageRank !== false)
        {
            $c->add("pageRank", $pageRank);
        }
        
        $c->setLimit(Config::get('maxTopRankCount'));
        
        return $this->selectWithNewFlag($c, "siteId, sites.url, imageSrc, siteTitle, description, pageRank, countryCode");     
    }
    
    function getValidatedTopReferrersSites()
    {
        $c = new Criteria();
        $c->add("referrerTimes", 0, ">");
        $c->add("status", "validated");
        $c->addOrder("referrerTimes DESC");
        $c->setLimit(Config::get('maxTopReferrersCount'));
        
        return $this->selectWithNewFlag($c, "siteId, sites.url, imageSrc, siteTitle, description, referrerTimes, countryCode");     
    } 
    
    function getSiteWithDetails($siteId)
    {
        $c = new Criteria();
        $c->add("siteId", $siteId);
        $sites = $this->selectWithNewFlag($c);
        return $sites ? $sites[0] : false;
    }
    
    function getValidatedSitesInCategory($categoryId, $page = false)
    {
        $c = new Criteria();
        $c->add("categoryId", $categoryId);
        $c->add("status", "validated");
        $c->addOrder("priority DESC");
        if($page !== false)$this->addPageCriteria($c, $page, Config::get("sitesPerPageInCategory"));
        
        return $this->selectWithNewFlag($c);
    }

    function createSearchCriteria($phrase)
    {
        $phrase = addslashes($phrase);
        $c = new Criteria();
        $c->add("MATCH (siteTitle, description)
        AGAINST ('$phrase' IN BOOLEAN MODE)");
        $c->addOr("LOCATE( '$phrase', url ) > 0 ");
        
        return $c; 
    }
    
    function addPageCriteria($c, $page, $resultsPerPage)
    {
        if(empty($page) || $page < 1)$page = 1;
        $c->setLimit($resultsPerPage);
        $c->setOffset(($page - 1) * $resultsPerPage);   
    }
    
    function search($phrase, $page = false)
    {
        $c = $this->createSearchCriteria($phrase);
        if($page !== false)$this->addPageCriteria($c, $page, Config::get("sitesPerPageInSearch"));
        return $this->selectWithNewFlag($c);
    }
    
    function searchValidated($phrase, $page = false)
    {
        $c = $this->createSearchCriteria($phrase);
        $c->add("status", "validated");
        if($page !== false)$this->addPageCriteria($c, $page, Config::get("sitesPerPageInSearch"));
        return $this->selectWithNewFlag($c);
    }
    
    function getSearchValidatedCount($phrase)
    {
        $c = $this->createSearchCriteria($phrase);
        $c->add("status", "validated");
        return $this->getCount($c); 
    }   
    
    function getSitesThatOwnAKeyword($keywordId, $page = false)
    {
        $c = new Criteria();
        $c->add("status", "validated");
        $c->addInnerJoin("keywordsofsites", "keywordsofsites.siteId", "sites.siteId");
        $c->add("keywordsofsites.keywordId", $keywordId);
        if($page !== false)$this->addPageCriteria($c, $page, Config::get("sitesPerPageInKeywords"));
        
        return $this->selectWithNewFlag($c);
    }
    
    function update($data, Criteria $c)
    {
        parent::update($data, $c);
        
        //if status changed must recalculate validated, refused counts
        if(isset($data['status']))
        {
            $this->updateStats();
        }        
    }
    
    function del(Criteria $c, $updateStats = true)
    {
        $condition = $c->getCondition("siteId");
        
        $nc = new Criteria();
        $nc->addCondition($condition);
        
        parent::del($c);
        
        Model::factoryInstance("keywordsOfSite")->del($nc);
        Model::factoryInstance("comment")->del($nc, false);
        Model::factoryInstance("vote")->del($nc);
        Model::factoryInstance("hit")->del($nc);
        
        if($updateStats)
        {
            $this->updateStats();
        }
    }
    
    function insert($data)
    {
        parent::insert($data);
        $this->updateStats();
    }
    
    function updateStats()
    {
        $this->category->updateValidatedSitesCount();
        Cacher::getInstance()->clean("tag", array("site", "category"));   
    }
    
    function validate($newSite, $oldSite = null, $forceDuplicate = false)
    {
        $categoryId = intval($newSite->categoryId);
        $url = $newSite->url;
        $isAdmin = (Session::getInstance()->get("role") == "administrator");
        
        // required fields validation
        if(empty($newSite->siteTitle) || empty($newSite->url) || empty($newSite->description))
            return 'Please, fill in the fields URL of site, Title and Description.';

        if(empty($categoryId))
            return 'Please, select a category from the dropdown list.';
            
        // ensure that site is not registered in this category
        if((!$oldSite || $oldSite->categoryId != $categoryId) && !$forceDuplicate && $this->whetherSiteIsRegisteredInCategory($url, $categoryId))
            return 'The site is already registered in this category.';
            
        // check whether the URL is allowed to be offered 
        if(!ereg('^http://', $url))
            return 'The URL must start with http://';
        
        if($this->getIfHostIsBanned($this->getHostNameFromUrl($url), 0))
            return 'This site is not allowed to be offered.';
            
        $minSiteDescriptionLength = Config::get("minSiteDescriptionLength");
            
        if($minSiteDescriptionLength && strlen(utf8_decode($newSite->description)) < $minSiteDescriptionLength && !$isAdmin)
        {
            return 'Description of site must have minimum '.$minSiteDescriptionLength.' characters length.';  
        }    
        
        if(!$oldSite && !$isAdmin)
        {
            if((integer)($this->getCountOfSiteCopiesInDifferentCategories($url)) >= (integer)(Config::get('maxCategoriesCountPerSite')))
                return 'This site is not allowed to be offered more times.';
        }
            
        if(Config::get('inscriptionCheckHttpResponseCode') == 1)
        {
            $httpClient = new HttpClient();
            $httpCode = $httpClient->checkResponseCodeOfSite($url);
            
            if($httpCode != 200)
                return 'This site\' HTTP response code is not 200. The site is not accepted.';
        }
        
        if(Config::get('backLinkMandatory') && empty($newSite->returnBond) && !$isAdmin)
        {
            return 'Backlink is mandatory.'; 
        }
        
        if(Config::get('countryFlagsEnabled') && empty($newSite->countryCode) && !$isAdmin)
        {
            return 'Country is mandatory.'; 
        }
         
        $category = $this->category->findByPk($categoryId);
        
        if(!$category || $category->possibleTender != 1)
            return 'Sites cannot be offered for this category.';
            
        if($this->bannedEmail->isBanned($newSite->webmasterEmail) && !$isAdmin)
            return 'This email is banned.';
            
        if($this->bannedIp->isBanned(Request::getInstance()->getIp()))
            return 'Your IP is banned.';
         
        return '';
    }
}

class SiteRecord extends ModelRecord
{
    function updateImageSrc()
    {
        $this->imageSrc = $this->getThumbnailSrc();
    }
              
    function getKeywordIds()
    {
        return $this->getKeywords(true);    
    }
    
    function getSimilarSites()
    {                 
        $c = new Criteria();
        $c->add("categoryId", $this->categoryId);
        $c->add("siteId", $this->siteId, "!=");
        $c->add("status", "validated");
        
        if(Config::get("similarSiteKeywordMatch"))
        {
            if(empty($this->keywords))return array();
            $keywordIds = implode(",", array_keys($this->keywords));
            $prefix = Config::get("DB_PREFIX"); 
            
            $c->add("EXISTS(SELECT * FROM ".$prefix."keywordsofsites
                     WHERE ".$prefix."keywordsofsites.siteId=".$prefix."sites.siteId AND keywordId IN ($keywordIds))");
        }
        
        $c->addOrder("RAND()");
        $c->setLimit(Config::get("randomSitesInDetailsCount"));
                
        return Model::factoryInstance("site")->findAll($c, "siteId, siteTitle, description");   
    }
    
    function getKeywords($onlyIds = false)
    {
        $c = new Criteria();
        $c->add("siteId", $this->siteId);
        
        if($onlyIds)
        {
            $field = "keywordId";
        }
        else
        {
            $field = "keywords.keywordId, keyword";
            $c->addInnerJoin("keywords", "keywords.keywordId", "keywordsofsites.keywordId");
        }
        
        $values = array();
        
        foreach(Model::factoryInstance("keywordsOfSite")->findAll($c, $field) as $keyword)
        {
            if($onlyIds)
            {
                $values[] = $keyword['keywordId'];
            }
            else
            {
                $values[$keyword['keywordId']] = $keyword['keyword']; 
            }
        }
        
        return $values;  
    }
    
    function downloadAndCacheThumb($forced = false)
    {
        $imageFileName = Config::get("SITES_THUMBS_PATH"). $this->siteId . ".jpg";
        
        if(!file_exists($imageFileName) || $forced)
        {
            if(!Config::get("thumbsGeneratorUrl"))return false;
            $url = Config::get("thumbsGeneratorUrl") . $this->url;
            $httpClient = new HttpClient();
            $thumbImage = $httpClient->getSiteContent($url, false, true, true);
            
            if($thumbImage)
            {
                file_put_contents($imageFileName, $thumbImage);
                return true;
            }
            else
            {
                return false;
            }
        }
        
        return true;
    }
    
    function getDefaultImageSrc()
    {
        return FrontController::getResourceUrl("/uploads/images_thumbs/default.jpg");  
    }   
    
    function getThumbnailSrc()
    {        
        $siteId = $this->siteId;

        if($this->imageSrc)
        {
            return FrontController::getResourceUrl("/uploads/images_thumbs/" . $this->imageSrc);
        }
        else
        {
            if(Config::get('cacheSiteImagesEnabled') == 1)
            {  
                $fileName = $siteId . '.jpg'; 
                $filePath = Config::get("SITES_THUMBS_PATH") . $fileName;
                
                if(file_exists($filePath))
                {
                    return FrontController::getResourceUrl("/uploads/images_thumbs/" . $fileName); 
                }
                else
                {
                    return FrontController::getRewrittedUrl("/site/getThumb/".$siteId); 
                }
            }            
            else
            {
                return Config::get("thumbsGeneratorUrl") . $this->url;
            }
    
        }

    }

}

?>
