<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////
    
class TemplateLiteView extends View 
{
    private $TemplateLite;
    private $viewFile;
    private $vars;
    
    private $adsLoaded = false;
    private $adsOnPage;
    private $adCriterias;
    private $predefinitions;
    private $pageIsPredefinied;
    private static $translate;
    
    function __construct($controller)
    {                               
        parent::__construct($controller);
        require_once(Config::get("COMPONENTS_PATH")."template_lite/class.template.php");  
        $this->TemplateLite = new Template_Lite();  
    }
     
    function modifierUrl()
    {
        $string = "";
        $args = func_get_args();
        $numArgs = func_num_args();
        
        for($i = 1; $i < $numArgs; $i++)
        {
            $args[$i] = str_replace("/", " ", $args[$i]);
        }
        
        $string = call_user_func_array('sprintf', $args);  
        
        return FrontController::getRewrittedUrl($string);
    }
    
    function modifierResUrl($string)
    {
        return FrontController::getResourceUrl($string);
    }  
    
    function modifierAction($string)
    {
        return FrontController::getInstance()->dispatch($string, true);
    }
    
    public static function preFilterLangMatchCallback($matches)
    {
        $string = $matches[1];
        $translate = self::$translate;
        return $translate->getPhrase($string);
    }
    
    public static function preFilterLang($templateContent)
    {       
        self::$translate = Translate::getInstance(Config::get("language"));
           
        $pattern = '#\{\'([^\']+)\'\|lang\}#';
                                             
        $templateContent = preg_replace_callback($pattern, array("TemplateLiteView", "preFilterLangMatchCallback"), $templateContent); 
        
        return $templateContent;
    }
    
    function highlight($string, $keywords)
    {         
        $keywordsMatch = '';

        foreach($keywords as $keyword)  
        {
            if($keywordsMatch != '')$keywordsMatch .= "|";
            $keywordsMatch .= str_replace('\*', '\w*', preg_quote($keyword, '#'));
        }

        $string = preg_replace('#(' . $keywordsMatch . ')#iu', '<b>\1</b>', $string); 

        return $string;
    }
    
    function domain($string)
    {         
        $urlParts = parse_url($string);
        $string = isset($urlParts['host']) ? $urlParts['host'] : $string; 
                          
        return $string;
    }
    
    function saveReferrer($params, &$tpl)
    {        
        $refererUrl = Request::getInstance()->getRefererUrl();
        
        if($refererUrl
           && FrontController::getHostNameFromUrl($refererUrl)
           != FrontController::getHostNameFromUrl(Config::get('siteRootUrl')))
        {
             Model::factoryInstance("otherReferrerSite")->saveReferrer($refererUrl);
        }
    }
    
    function displayAd($params, &$tpl)
    {
        $place = $params['place'];
        $page = Display::get("adPage");
        if(empty($page))return ""; 
        
        if(!$this->adsLoaded)
        {
            $page = Display::get("adPage");
            
            $cache = Cacher::getInstance(); 
            
            //get Ads ids on current page
            if(($data = $cache->load("adsOnPage".$page)) === null)
            {       
                $data = Model::factoryInstance("ad")->getAdsOnPage($page); 
                $cache->save($data, null, null, array("adcriteria", "ad"));
            }
            
            $this->adsOnPage = $data;
            
            //load predefined settings
            if(($data = $cache->load("adsPredefinitions")) === null)
            {       
                $data = Model::factoryInstance("ad")->getAllPredefinitions();
                $cache->save($data, null, null, array("adcriteria", "ad"));
            }
            
            $this->predefinitions = $data;
            
            //load ads html content
            if(($data = $cache->load("adsAllCriterias")) === null)
            {       
                $data = Model::factoryInstance("adCriteria")->generateList(null, "htmlContent"); 
                $cache->save($data, null, null, array("adcriteria"));
            }
            
            $this->adCriterias = $data;
 
            $this->adsLoaded = true;
            
            $this->pageIsPredefinied = (!empty($this->adsOnPage['predefine'])
            || (substr_compare($page, "site", 0, 4) == 0
                && Model::factoryInstance("ad")->isSiteAdsPredefinedInCategory(Display::get("adSiteCategoryId")) 
               )
            ); 
        }
        
        $htmlContent = "";        
        
        if(!empty($this->adsOnPage['general']) && !empty($this->adsOnPage[$place]))
        {
            $adCriterionId = $this->adsOnPage[$place];
            
            if(!empty($this->adCriterias[$adCriterionId]))
            {
                $htmlContent = $this->adCriterias[$adCriterionId];
            }
        }
        else
        {
            if($this->pageIsPredefinied && (!isset($this->adsOnPage['general']) || $this->adsOnPage['general'] == 1 ))
            {                
                foreach(array("site", "category", "keyword") as $typeName)
                {
                    if(substr_compare($typeName, $page, 0, strlen($typeName)) == 0
                    && !empty($this->predefinitions[$typeName][$place]))
                    {
                        $adCriterionId = $this->predefinitions[$typeName][$place];
                        
                        if(!empty($this->adCriterias[$adCriterionId]))
                        {
                            $htmlContent = $this->adCriterias[$adCriterionId];
                        }                   
                    }
                }
            }    
        }
        
        return $htmlContent;
    }
    
    function displayTime($params, &$tpl)
    {
        $script_end_time = microtime(true);
        return "&nbsp; &nbsp; Generated in ".sprintf("%.3f", $script_end_time - $GLOBALS['script_start_time'])." Queries: ".Display::get("queriesCount", 0);
    }
    
    function render()
    {
        $tpl = $this->TemplateLite; 
        
        $tpl->register_prefilter('cacheLangs');
                
        $templateName = (!empty($_COOKIE['previewTemplateName']) && Session::getInstance()->get("role") == "administrator") ? $_COOKIE['previewTemplateName'] : Config::get("templateName");
        
        $tpl->template_dir = CODE_ROOT_DIR.$this->controller->localDir."templates/".$templateName;
        
        if(!is_dir($tpl->template_dir))
        {
            $templateName = Config::get("DEFAULT_TEMPLATE_NAME");
            $tpl->template_dir = CODE_ROOT_DIR.$this->controller->localDir."templates/".$templateName;
        }
        
        $tpl->compile_dir = CODE_ROOT_DIR."compiled/".$this->controller->localDir.$templateName."_template"."/".Config::get("language");  
         
         if(!is_dir($tpl->compile_dir))
         {
             $currentPath = CODE_ROOT_DIR."compiled/";
              
             $dirs = explode("/", substr($tpl->compile_dir, strlen($currentPath)));
             umask(0);
             
             foreach($dirs as $dir)
             {
                 $currentPath .= $dir."/";
                 if(!is_dir($currentPath))
                 {
                     mkdir($currentPath);
                 }          
             }
         }

        //$tpl->assign("lang", array("Directory" => "Dire"));
        $tpl->assign("setting", $this->arrayMapRecursive("utf8_htmlspecialchars", Config::getAll()));   
        $tpl->assign("display", Display::getAll());
        
        $tpl->assign("controllerAction", $this->controller->action);
        $tpl->assign("controllerName", $this->controller->name);
        
        $tpl->assign($this->arrayMapRecursive("utf8_htmlspecialchars", $this->controller->viewVars));
        
        $tpl->assign("action", $this->controller->action); 
        $tpl->assign("session", Session::getInstance()->toArray()); 
        
        $tpl->register_modifier("action", array($this, "modifierAction"));
        $tpl->register_modifier("url", array($this, "modifierUrl")); 
        $tpl->register_modifier("resurl", array($this, "modifierResUrl")); 
        $tpl->register_modifier("highlight", array($this, "highlight"));
        $tpl->register_modifier("domain", array($this, "domain"));
      
        $tpl->register_function("displayAd", array($this, "displayAd"));
        $tpl->register_function("display_time", array($this, "displayTime"));
        $tpl->register_function("saveReferrer", array($this, "saveReferrer"));
        
        if($this->controller->viewFile)
        {
            $view_file = NameTool::getTemplateFileName(strtolower($this->controller->niceName)."/".$this->controller->viewFile);
        }
        else
        {
            $view_file = NameTool::getTemplateFileName(strtolower($this->controller->niceName)."/".$this->controller->action); 
        }
        
        $tpl->encode_file_name = false;
        return $tpl->fetch($view_file);
    }
}

?>